/* ox_sound.c  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <linux/soundcard.h>

#define BUFSIZE 44100



/* WAVE $B%U%!%$%k$N>pJs$r3JG<$9$k9=B$BN(B */
typedef struct {
	FILE* fp;        /* $B%U%!%$%k9=B$BN(B */
	short is_pcm;    /* PCM $B%U%)!<%^%C%H$N>l9g$O(B 1$B!"$=$l0J30$O(B 0 */
	short channel;   /* $B%b%N%i%k$N>l9g$O(B 1$B!"%9%F%l%*$N>l9g$O(B 2 */
	int   rate;      /* $B%5%s%W%j%s%0<~GH?t(B */
	short bits;      /* $BNL;R2=%S%C%H?t(B */
	long  offset;    /* $B%U%!%$%k@hF,$+$i(B PCM $B%G!<%?$^$G$N%*%U%;%C%H(B */
	int   len;       /* PCM $B%G!<%?It$ND9$5(B */
}
WAVE;


/* $B4X?t%W%m%H%?%$%W(B */
static int  wave_read_file( char* fname, WAVE* wave );
static int  wave_setup_dsp( int* dsp, WAVE* wave );
static void wave_progress( int* processed, int current, WAVE* wave );
int play( char* s);





int play(char *s){

  WAVE wave;
  int len,fd;
  FILE *fp;
  char buf[BUFSIZE];
  int processed = 0;
  
  if (wave_read_file( s , &wave ) !=0){
    return -1;
  }
  else {
    
    printf( "\n" );
    printf( " Input Filename : %s\n",  s);
    printf( " Sample Size    : %d\n",  wave.bits );
    printf( " Channels       : %d\n",  wave.channel );
    printf( " Sample Rate    : %d\n",  wave.rate );
    printf( " Offsets        : %ld\n", wave.offset );
    printf( " Sample Length  : %d\n",  wave.len );
    printf( "\n" );



    if(wave_setup_dsp( &fd, &wave ) != 0){
      fprintf(stderr,"setup/dev/dsp failed.\n");
      close(fd);
      return -1;
    }

    printf( "Now playing" );
  
    fseek( wave.fp, wave.offset, SEEK_SET);
    wave_progress( &processed, 0,&wave);
  
    

    while(1){
      len = fread(buf, sizeof(char), BUFSIZE, wave.fp);
      if(feof(wave.fp)!=0){
	if(write(fd,buf,len) == -1){
	  perror("write()");
	}
	else {
	  wave_progress( &processed,len, &wave);
	}
	break;
      }else{
	if (write(fd,buf,BUFSIZE) ==-1){
		perror("write()");
		break;
	  }
	  wave_progress( &processed, len, &wave );
	}
 }

 fclose(wave.fp);
 close(fd);
 return 0;
	
  }
}
 
/*
 * WAVE $B%U%!%$%k>pJs$rFI$_9~$`(B
 */
static int 
wave_read_file( char* fname, WAVE* wave )
{
	char   buf[32];
	int    len;

	if ( ( wave->fp = fopen( fname, "rb" ) ) == NULL ) {
		fprintf( stderr, "Failed to open %s\n", fname );
		return -1;
	}

	/* 
	 * $B@hF,(B 4 $B%P%$%H$,(B "RIFF" $B$G$"$k$3$H$r3NG'(B
	 * $B99$K(B 4 $B%P%$%H%9%-%C%W$7$F$*$/(B
	 */
	fread( buf, 8, 1, wave->fp );

	if ( strncmp( buf, "RIFF", 4 ) != 0 ) {
		fprintf( stderr, "Specified file is not RIFF file.\n" );
		fclose( wave->fp );
		return -1;
	}


	/* $B<!$N(B 4 $B%P%$%H$,(B "WAVE" $B$G$"$k$3$H$r3NG'(B */
	fread( buf, 4, 1, wave->fp );

	if ( strncmp( buf, "WAVE", 4 ) != 0 ) {
		fprintf( stderr, "Specified file is not WAVE file.\n" );
		fclose( wave->fp );
		return -1;
	}

	/* fmt $B%A%c%s%/$rC5$9(B */
	while ( 1 ) {
		fread( buf, 8, 1, wave->fp );
		len = *( int* )( &buf[4] );

		if ( strncmp( buf, "fmt ", 4 ) != 0 ) {
			if ( fseek( wave->fp, len, SEEK_CUR ) == -1 ) {
				fprintf( stderr, "Failed to find fmt chunk.\n" );
				fclose( wave->fp );
				return -1;
			}
		}
		else {
			break;
		}
	}


	/* WAVE $B%U%)!<%^%C%H$rFI$_9~$`(B */
	fread( buf, len, 1, wave->fp );

	wave->is_pcm  = *( ( short* )( &buf[0]  ) );
	wave->channel = *( ( short* )( &buf[2]  ) );
	wave->rate    = *( ( int*   )( &buf[4]  ) );
	wave->bits    = *( ( short* )( &buf[14] ) );

	if ( wave->is_pcm != 1 ) {
		wave->is_pcm = 0;
	}

	/* data $B%A%c%s%/$rC5$9(B */
	while ( 1 ) {
		fread( buf, 8, 1, wave->fp );
		len = *( int* )( &buf[4] );

		if ( strncmp( buf, "data", 4 ) != 0 ) {
			if ( fseek( wave->fp, len, SEEK_CUR ) == -1 ) {
				fprintf( stderr, "Failed to find data chunk.\n" );
				fclose( wave->fp );
				return -1;
			}
		}
		else {
			break;
		}
	}

	wave->len = len;

	if ( ( wave->offset = ftell( wave->fp ) ) == -1 ) {
		fprintf( stderr, "Failed to find offset of PCM data.\n" );
		fclose( wave->fp );
		return -1;
	}

	return 0;
}



/*
 * /dev/ dsp $B$r@_Dj$9$k(B
 */
static int 
wave_setup_dsp( int* dsp, WAVE* wave )
{
	int format;
	int rate;
	int channel;

	/*
	 * PCM $B%U%)!<%^%C%H$G$"$k$+$I$&$+3NG'$9$k(B
	 */
	if ( ! wave->is_pcm ) {
		fprintf( stderr, "Specified file's sound data is not PCM format.\n" );
		fclose( wave->fp );
		return -1;
	}

	/*
	 * 8 bit $B$N>l9g$O(B AFMT_U8$B!"(B16 bit $B$N>l9g$O(B AFMT_S16_LE $B$r%U%)!<%^%C%H(B
	 * $B$H$7$FA*Br$7!"$=$l0J30$N?t$N>l9g$O%(%i!<$H$9$k!#(B
	 */
	if ( wave->bits == 8 ) {
		format = AFMT_U8;
	}
	else if ( wave->bits == 16 ) {
		format = AFMT_S16_LE;
	}
	else {
		fprintf( stderr, "Specified file's sound data is %d bits,"
			 "not 8 nor 16 bits.\n", wave->bits );
		fclose( wave->fp );
		return -1;
	}

	rate    = ( int )wave->rate;
	channel = ( int )wave->channel;


	if ( ( *dsp = open( "/dev/dsp", O_WRONLY ) ) == -1 ) {
		perror( "open()" );
		return -1;
	}

	if ( ioctl( *dsp, SNDCTL_DSP_SETFMT, &format ) == -1 ) {
		perror( "ioctl( SOUND_PCM_SETFMT )" );
		close( *dsp );
		return -1;
	}

	if ( ioctl( *dsp, SOUND_PCM_WRITE_RATE, &rate ) == -1 ) {
		perror( "ioctl( SOUND_PCM_WRITE_RATE )" );
		close( *dsp );
		return -1;
	}

	if ( ioctl( *dsp, SOUND_PCM_WRITE_CHANNELS, &channel ) == -1 ) {
		perror( "ioctl( SOUND_PCM_WRITE_CHANNELS )" );
		close( *dsp );
		return -1;
	}

	return 0;
}



/*
 * $B?JD=N($NI=<((B
 */
static void 
wave_progress( int* processed, int current, WAVE* wave )
{
	int progress;

	*processed += current;
	progress = (int)( ( ( double )*processed / ( double )wave->len ) * 100 );
	printf( "\r%3d%% played.", progress );
	fflush( stdout );
}



/* ox_sound.c  end */
